--[[--------------------
--
-- MassMailer by Shirokuma
-- 
-- Licensed under the zlib/libpng License
--
-- Copyright (c) <2010> <Bryan Turley>
--
-- This software is provided 'as-is', without any express or implied
-- warranty. In no event will the authors be held liable for any damages
-- arising from the use of this software.
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictions:
--
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software. If you use this software
--    in a product, an acknowledgment in the product documentation would be
--    appreciated but is not required.
--
-- 2. Altered source versions must be plainly marked as such, and must not be
--    misrepresented as being the original software.
--
-- 3. This notice may not be removed or altered from any source
--    distribution.
--
--]]--------------------

--[[--------------------
-- Set Up Ace3 Libraries
--]]--------------------
local MM = LibStub("AceAddon-3.0"):NewAddon("MassMailer", "AceEvent-3.0", "AceConsole-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MassMailer", true)

--[[--------------------
-- Local Variables
--]]--------------------
-- functions
local pairs, tinsert, tremove = pairs, table.insert, table.remove

-- frames
local UpdateFrame = CreateFrame("Frame")
local frames = {}
local eventbtns = {}
local mailclosed = true
local mmMiniMapButton = nil

-- player info
local playername = nil
local guildinfo = {
	["rank"] = {}, 
	["member"] = {},
}
local friends = {}
local arenainfo = {
	["2v2"] = {},
	["3v3"] = {},
	["5v5"] = {},
}

-- saved options
local loadedoptions = {}
local loadedgroups = {
	["names"] = {},
	["members"] = {},
}
local loadeddrafts = {}
local loadedsigs = {}

-- message variables
local message = {
	active = false,
	ready = false,
	errors = {},
}

--[[--------------------
-- Support functions
--]]--------------------
-- unchecks all the other checkboxes in on a frame
local CheckOthers = function(list, widget, value)
	for _, item in pairs(list) do
		-- Bah, I hate calling undocumented stuff, but I
		-- don't know of any other way to see if an item is disabled.
		-- Guess I'll submit a ticket to the Ace3 folks...
		if item ~= widget and item.disabled ~= true then  
			item:SetValue(value)
		end
	end
end

-- split and trim methods taken from the LUA documentation...it's easier than writing my own
-- http://lua-users.org/wiki/StringRecipes
local SplitString = function (str, delim, maxNb) 
    if string.find(str, delim) == nil then
        return { str }
    end
    if maxNb == nil or maxNb < 1 then
        maxNb = 0
    end
    local result = {}
    local pat = "(.-)" .. delim .. "()"
    local nb = 0
    local lastPos
    for part, pos in string.gmatch(str, pat) do
        nb = nb + 1
        result[nb] = part
        lastPos = pos
        if nb == maxNb then break end
    end
    if nb ~= maxNb then
        result[nb + 1] = string.sub(str, lastPos)
    end
    return result
end

local TrimString = function (s)
  return s:match'^%s*(.*%S)' or ''
end

-- takes a comma separated string and returns a table of strings
local BreakString = function(str)
	local strtable = SplitString(str, ",")
	
	for index, value in pairs (strtable) do
		strtable[index] = TrimString(value)
	end

	return strtable
end

-- takes the message body text and returns a table of strings in 500 character sections
-- tries to avoid splitting words in half (looks for spaces)
local ParseBody = function(str) 
	local textLength = str:len()
	local bodyTable = {}
	if (textLength > 500) then
		local firstPos = 1
		local lastPos = 500
		local i = 1
		
		repeat
			local offSet = 0
			local j = lastPos
			
			repeat
				j = j - 1
			until (str:sub(j,j) == " " or j <= 0)
			
			if not (j <= 0) then
				offSet = lastPos - j
				lastPos = j
			end
									
			bodyTable[i] = str:sub(firstPos, lastPos)
			firstPos = firstPos + 500 - offSet
			lastPos = lastPos + 500
			
			i = i + 1
			
			if (lastPos >= textLength) then 
				lastPos = textLength
				bodyTable[i] = str:sub(firstPos, lastPos)
			end
		until (lastPos == textLength)

		return bodyTable
	else
		bodyTable[1] = str
		return bodyTable
	end
end

-- load the saved options
function MM:LoadOptions()
	loadedoptions = {}
	loadeddrafts = {}
	loadedsigs = {}
	loadedgroups["names"] = {}
	loadedgroups["members"] = {}
	
	for option, value in pairs (self.db.profile.options) do
		loadedoptions[option] = value
	end
	
	for name, list in pairs (self.db.profile.groups) do
		tinsert(loadedgroups["names"], name)
		tinsert(loadedgroups["members"], list)
	end
	
	for title, draft in pairs (self.db.profile.drafts) do
		loadeddrafts[title] = draft
	end
	
	for title, sig in pairs (self.db.profile.signatures) do
		loadedsigs[title] = sig
	end		
end

-- opens the massmailer interface options window
function MM:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory(MM.optionspanel.profiles)
	InterfaceOptionsFrame_OpenToCategory(MM.optionspanel)
end

-- preforms actions based on /slash input
function MM:ProcessSlashCommand(input)
	if input == L["config"] then
		MM:OpenOptions()
	elseif input == L["open"] then
		MM:ToggleMessageFrame()
	elseif input == L["help"] or "" then
		print ("|cFFFFEE33"..L["MassMailer"].." - "..L["Slash Commands"]..":|r")
		print ("|cFFFFEE33 /mm "..L["config"].."|r - "..L["Open the MassMailer options window"])
		print ("|cFFFFEE33 /mm "..L["open"].."|r - "..L["Open the MassMailer message window"])
		print ("|cFFFFEE33 /mm "..L["help"].."|r - "..L["Display this slash command listing"])
	end
end

-- creates a minimap button and sets up button actions
function MM:LoadMiniMapButton()
	mmMiniMapButton = CreateFrame("Button", nil, Minimap)
	mmMiniMapButton:SetFrameStrata("LOW")
	mmMiniMapButton:SetSize(33, 33)
	mmMiniMapButton:EnableMouse(true)
	mmMiniMapButton:SetMovable(true)
	mmMiniMapButton:SetPoint("TOPLEFT")
	mmMiniMapButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	mmMiniMapButton:RegisterForDrag("LeftButton", "RightButton")
	
	-- don't show if button is disabled
	if not loadedoptions["minimap"].show then
		mmMiniMapButton:Hide()
	end
	
	local mmMiniMapButtonTexture = mmMiniMapButton:CreateTexture(nil, "ARTWORK")
		mmMiniMapButtonTexture:SetSize(21, 21)
		mmMiniMapButtonTexture:SetPoint("TOPLEFT", 7, -6)
		mmMiniMapButtonTexture:SetTexture("Interface\\MailFrame\\Mail-Icon")
	local mmMiniMapButtonBorder = mmMiniMapButton:CreateTexture(nil, "OVERLAY")
		mmMiniMapButtonBorder:SetSize(56, 56)
		mmMiniMapButtonBorder:SetPoint("TOPLEFT")
		mmMiniMapButtonBorder:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
	local mmMiniMapButtonHighlight = mmMiniMapButton:CreateTexture(nil, "HIGHLIGHT")
		mmMiniMapButtonHighlight:SetTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
		mmMiniMapButtonHighlight:SetBlendMode("ADD")
		mmMiniMapButtonHighlight:SetSize(33, 33)	
		mmMiniMapButtonHighlight:SetPoint("TOPLEFT")
			
	local mmMiniMapButtonUpdateFrame = CreateFrame("Frame")
		mmMiniMapButtonUpdateFrame:Hide()
	
	-- assistance with the math from Gello @ WoWInterface
	-- http://www.wowinterface.com/forums/showthread.php?p=164066
	local mmRepositionMinimapButton = function()
		mmMiniMapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(loadedoptions["minimap"].pos)),(80*sin(loadedoptions["minimap"].pos))-52)
	end
			
	mmMiniMapButtonUpdateFrame:SetScript("OnUpdate", function()
		local xpos,ypos = GetCursorPosition()
		local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

		xpos = xmin-xpos/UIParent:GetScale()+70
		ypos = ypos/UIParent:GetScale()-ymin-70

		loadedoptions["minimap"].pos = math.deg(math.atan2(ypos,xpos))
		self.db.profile.options.minimap.pos = loadedoptions["minimap"].pos
		mmRepositionMinimapButton()
	end)
	
	mmMiniMapButton:SetScript("OnDragStart", function(this)
		this:LockHighlight()
		this.dragging = true
		mmMiniMapButtonUpdateFrame:Show()
	end)
	mmMiniMapButton:SetScript("OnDragStop", function(this)
		this:UnlockHighlight()
		this.dragging = false
		mmMiniMapButtonUpdateFrame:Hide()
	end)
	mmMiniMapButton:SetScript("OnClick", function(this, button)
		if button == "LeftButton" then
			MM:ToggleMessageFrame()
		elseif button == "RightButton" then
			MM:OpenOptions()
		end
	end)
	mmMiniMapButton:SetScript("OnEnter", function(self)
		if self.dragging then return end
		GameTooltip:SetOwner(self or UIParent, "ANCHOR_LEFT")
		GameTooltip:AddLine(L["MassMailer"])
		GameTooltip:AddLine("|cFF33EEFF"..L["Left Click"].."|r: "..L["Open Message window"])
		GameTooltip:AddLine("|cFF33EEFF"..L["Right Click"].."|r: "..L["Open MassMailer options"])
		GameTooltip:Show()
	end)
	mmMiniMapButton:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	
	-- fix pos on load
	mmRepositionMinimapButton()
end

-- creates a tooltip with the give title and body
function MM:ShowTooltip(title, str)
	if title or str then
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
		GameTooltip:SetText(title)
		GameTooltip:AddLine(str, 1, 1, 1)
		GameTooltip:Show()
	else
		GameTooltip:Hide()
	end
end

--[[--------------------
-- OnInitialize
--]]--------------------
function MM:OnInitialize()
	-- set up default options
	local defaults = {
		profile = {
			groups = {},
			options = {
				insertsig = false,
				defaultsig = "",
				frameclamp = true,
				showtooltips = true,
				delay = true,
				autoclose = true,
				autoclear = false,
				minimap = {
					show = true,
					pos = 45,
				},
				officerranks = {
					["Rank 0"] = true,
					["Rank 1"] = true,
					["Rank 2"] = false,
					["Rank 3"] = false,
					["Rank 4"] = false,
					["Rank 5"] = false,
					["Rank 6"] = false,
					["Rank 7"] = false,
					["Rank 8"] = false,
					["Rank 9"] = false,
				},
			},
			drafts = {},
			signatures = {},
		}
	}
	
	-- register saved variables
	self.db = LibStub("AceDB-3.0"):New("MassMailerDB", defaults)
	
	-- set callbacks for switching profiles
	self.db.RegisterCallback(self, "OnProfileChanged", "LoadOptions")
	self.db.RegisterCallback(self, "OnProfileCopied", "LoadOptions")
	self.db.RegisterCallback(self, "OnProfileReset", "LoadOptions")
	
	-- set up the options table for the interface options window
	local options = {
		type = "group",
		args = {
			config = {
				name = "Configuration",
				type = "group",
				get = function(info)
					return loadedoptions[info[#info]]
				end,
				set = function(info, value)
					self.db.profile.options[info[#info]] = value
					loadedoptions[info[#info]] = value
				end,
				args = {
					general = {
						type = "header",
						name = L["General Options"],
						order = 0,
					},
					frameclamp = {
						name = L["Frame Clamp"],
						desc = L["Enable / disable screen clamping, which prevents the MassMailer windows from appearing off-screen.  Useful for smaller screen resolutions."],
						descStyle = "inline",
						type = "toggle",
						order = 1,
					},
					delay = {
						name = L["Send Delay"],
						desc = L["Enable / disable delay between messages. Increases send time but ensures multi-part messages will be received in order."],
						descStyle = "inline",
						type = "toggle",
						order = 2,
					},
					showtooltips = {
						name = L["Show Tooltips"],
						desc = L["Enable / disable tooltips that explain the functions of different buttons."].."  "..L["Requires reload / relog to take effect."],
						descStyle = "inline",
						type = "toggle",
						order = 3,
					},
					autoclose = {
						name = L["Close on Mailbox exit"],
						desc = L["Automatically close the MassMailer windows when you exit your Mailbox."],
						descStyle = "inline",
						type = "toggle",
						order = 4,
					},
					autoclear = {
						name = L["Clear Message on Send"],
						desc = L["Automatically clear the address, subject, and body fields after sending a message."],
						descStyle = "inline",
						type = "toggle",
						order = 5,
					},
					minimap = {
						name = L["Show MiniMap Icon"],
						desc = L["Put a MassMailer icon on the Minimap."],
						descStyle = "inline",
						type = "toggle",
						order = 6,
						set = function(info, value)
							self.db.profile.options[info[#info]].show = value -- added .show
							loadedoptions[info[#info]].show = value -- added .show
							
							if mmMiniMapButton then -- end
								if value then
									mmMiniMapButton:Show()
								else
									mmMiniMapButton:Hide()
								end
							end
						end,
						get =function(info)
							return loadedoptions[info[#info]].show
						end,
					},
					social = {
						type = "header",
						name = L["Social Options"],
						order = 7,
					},
					officerranks = {
						type = "multiselect",
						name = L["Default Officer Ranks"],
						desc = L["Rank 0 = Guild Leader, Rank 9 = Lowest Rank."],
						values = function()
								local list = {}
								for i=0, 9, 1 do
									list[L["Rank"].." "..tostring(i)] = L["Rank"].." "..tostring(i)
								end
								return list
							end,
						order = 8,
						get = function(info, key)
							return loadedoptions[info[#info]][key]
						end,
						set = function(info, key, value)
							self.db.profile.options[info[#info]][key] = value
							loadedoptions[info[#info]][key] = value
						end,
					},
					officerdesc = {
						type = "description",
						name = L["Sets the default officer ranks.  Used in the default Officer group and the [Officers] group shortcut.  The rank numbers are found on the WoWArmory page for your guild.  The Guild Leader is always Rank 0, and the lowest possible rank is Rank 9.  Not all guilds use all 10 possible ranks."].."\n",
						fontSize = "medium",
						order = 9,
					},
					signatures = {
						type = "header",
						name = L["Signature Options"],
						order = 10,
					},
					insertsig = {
						type = "toggle",
						name = L["Include Signature"],
						desc = "Enable / disable automatic inclusion of signatures in new messages.",
						order = 11,
					},
					defaultsig = {
						type = "select",
						name = L["Default Signature"],
						desc = L["Select the default signature to include with new messages (if enabled)."],
						order = 12,
						values = function()
							local list = {}
							for title,value in pairs (loadedsigs) do
								list[title] = title
							end
							return list
						end,
						get = function(info)
							return loadedoptions[info[#info]]
						end,
						set = function(info, key)
							self.db.profile.options[info[#info]] = key
							loadedoptions[info[#info]] = key
						end,
					},
					sigdesc = {
						type = "description",
						name = L["Requires reload / relog to take effect."],
						fontSize = "medium",
						order = 13,
					},
				}
			}
		}
	}
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	-- register options table
	AceConfig:RegisterOptionsTable("MassMailer", options)
	MM.optionspanel = AceConfigDialog:AddToBlizOptions(L["MassMailer"], L["MassMailer"], nil, "config")
	MM.optionspanel.profiles = AceConfigDialog:AddToBlizOptions(L["MassMailer"], L["Profiles"], L["MassMailer"], "profiles")
	MM:RegisterChatCommand("massmailer", "ProcessSlashCommand")
	MM:RegisterChatCommand("mm", "ProcessSlashCommand")
end

--[[--------------------
-- OnEnable
--]]--------------------
function MM:OnEnable()
	-- set up player info
	playername,_ = UnitName("player")
	
	-- load saved options
	MM:LoadOptions()
	
	-- set up OnUpdate
	UpdateFrame:SetScript("OnUpdate", UpdateFrame.OnUpdate)
	
	-- create MassMailer button
	local mailButton = CreateFrame("Button",nil,SendMailFrame,"UIPanelButtonTemplate")
		local buttonText = mailButton:CreateFontString()
			buttonText:SetFont("Fonts\\FRIZQT__.TTF", 10, nil)
			buttonText:SetShadowColor(0,0,0,1)
			buttonText:SetShadowOffset(1,-1)
			buttonText:SetText(L["MassMailer"])
		mailButton:SetFontString(buttonText)
		mailButton:SetHeight(19)
		mailButton:SetWidth(74)
		mailButton:SetPoint("TOPRIGHT", -74,-15)
		mailButton:SetScript("OnClick",function() MM:ToggleMessageFrame() end)
		
		if loadedoptions["showtooltips"] then
			mailButton:SetScript("OnEnter", function() MM:ShowTooltip(L["MassMailer"], L["Open MassMailer"]) end)
			mailButton:SetScript("OnLeave", function() MM:ShowTooltip() end)
		end

	-- Register Events
	MM:RegisterEvent("MAIL_CLOSED")
	MM:RegisterEvent("MAIL_SHOW")
	MM:RegisterEvent("CALENDAR_OPEN_EVENT")
	MM:RegisterEvent("GUILD_ROSTER_UPDATE")
	MM:RegisterEvent("PLAYER_GUILD_UPDATE")
	MM:RegisterEvent("FRIENDLIST_UPDATE")
	MM:RegisterEvent("ARENA_TEAM_ROSTER_UPDATE")
	MM:RegisterEvent("MAIL_SEND_SUCCESS")
	MM:RegisterEvent("MAIL_FAILED")
	
	-- load player info
	if IsInGuild() then
		GuildRoster()
	end
	if GetNumFriends() > 0 then	
		ShowFriends()
	end
	if IsInArenaTeam() then
		if ArenaTeam_GetTeamSizeID(2) then
			ArenaTeamRoster(ArenaTeam_GetTeamSizeID(2))
		end
		if ArenaTeam_GetTeamSizeID(3) then
			ArenaTeamRoster(ArenaTeam_GetTeamSizeID(3))
		end
		if ArenaTeam_GetTeamSizeID(5) then
			ArenaTeamRoster(ArenaTeam_GetTeamSizeID(5))
		end
	end

	-- create minimap button
	MM:LoadMiniMapButton()
	
	-- print welcome message
	print ("|cFFFFEE33"..L["MassMailer"].."|r "..L["loaded"])
	print (L["Type /mm or /massmailer for more info."])
end

--[[--------------------
-- OnUpdate
--]]--------------------
function UpdateFrame.OnUpdate(self, elapsed)
	-- hide EventFrame on calendar close
	if (CalendarCreateEventFrame and CalendarViewEventFrame) then
		if not (CalendarCreateEventFrame:IsVisible() or CalendarViewEventFrame:IsVisible()) then
			if (frames.EventFrame) then
				frames.EventFrame:Hide()
			end
		end
	end
	
	-- send the next message
	-- Note: For whatever reason, calling SendMail() from inside a MAIL_SEND_SUCCESS or MAIL_FAILED event
	-- doesn't seem to work, so instead I have a variable that tracks if the events have fired.  Then OnUpdate
	-- checks the status of the variable.  If it's true, OnUpdate sends the next message.
	if message.ready then
		if loadedoptions["delay"] then
			if time() - message.time >= 1 then
				message.ready = false
				message.time = time()
				SendMail(message.names[message.niter], message.subject.." ("..message.biter.." / "..getn(message.body)..")", message.body[message.biter])
			end
		else
			message.ready = false
			message.time = time()
			SendMail(message.names[message.niter], message.subject.." ("..message.biter.." / "..getn(message.body)..")", message.body[message.biter])
		end
	end
end

--[[--------------------
-- Event Handlers
--]]--------------------
function MM:MAIL_CLOSED()
	if loadedoptions["autoclose"] then
		MM:CloseAllFrames()
	end
	mailclosed = true
	if frames.MessageFrame then
		frames.MessageFrame.sendbtn:SetDisabled(true)
	end
end

function MM:MAIL_SHOW()
	mailclosed = false
	if frames.MessageFrame then
		frames.MessageFrame.sendbtn:SetDisabled(false)
	end
end

function MM:CALENDAR_OPEN_EVENT()
	local _,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,calendarType = CalendarGetEventInfo()
	-- don't add a mail button to guild announcements (no members)
	if calendarType ~= "GUILD_ANNOUNCEMENT" then
		-- add a button for the CalendarCreatEventFrame
		if CalendarCreateEventFrame then
			if not eventbtns.createbtn then
				local createeventbtn = CreateFrame("Button", nil, CalendarCreateEventFrame, "UIPanelButtonTemplate")
					createeventbtn:SetText(L["Mail"])
					createeventbtn:SetWidth(80)
					createeventbtn:SetHeight(20)
					createeventbtn:SetScript("OnClick", function () MM:ToggleEventFrame() end)
					createeventbtn:SetPoint("TOPRIGHT", -16, -120)
					createeventbtn:SetScript("OnEnter", function() MM:ShowTooltip(L["Mail"], L["Open the event mailer window."]) end)
					createeventbtn:SetScript("OnLeave", function() MM:ShowTooltip() end)
					eventbtns.createbtn = createeventbtn
			else
				eventbtns.createbtn:Show()
			end
		end
		-- add a button for the CalendarViewEventFrame
		if CalendarViewEventFrame then
			if not eventbtns.viewbtn then
				local vieweventbtn = CreateFrame("Button", nil, CalendarViewEventFrame, "UIPanelButtonTemplate")
					vieweventbtn:SetText("Mail")
					vieweventbtn:SetWidth(80)
					vieweventbtn:SetHeight(20)
					vieweventbtn:SetScript("OnClick", function () MM:ToggleEventFrame() end)
					vieweventbtn:SetPoint("TOPRIGHT", -16, -70)
					vieweventbtn:SetScript("OnEnter", function() MM:ShowTooltip(L["Mail"], L["Open the event mailer window."]) end)
					vieweventbtn:SetScript("OnLeave", function() MM:ShowTooltip() end)
					eventbtns.viewbtn = vieweventbtn
			else
				eventbtns.viewbtn:Show()
			end
		end
	else
		-- hide any previously created buttons is a guild announcement is selected
		if eventbtns.createbtn then
			eventbtns.createbtn:Hide()
		end
		if eventbtns.viewbtn then
			eventbtns.viewbtn:Hide()
		end
		if frames.EventFrame then
			frames.EventFrame:Hide()
		end
	end
	-- enable / disable the "signed up" checkbox if applicable
	if calendarType == "GUILD_EVENT" then
		if frames.EventFrame then
			frames.EventFrame.checkboxes.signedupcheck:SetDisabled(false)
			frames.EventFrame.checkboxes.signedupcheck:SetValue(true)
		end
	else
		if frames.EventFrame then
			frames.EventFrame.checkboxes.signedupcheck:SetDisabled(true)
			frames.EventFrame.checkboxes.signedupcheck:SetValue(false)
		end
	end
end

function MM:GUILD_ROSTER_UPDATE()
	-- index the guild members if you are in a guild
	if IsInGuild() then
		guildinfo["rank"] = {}
		guildinfo["member"] = {}
		for i=1, GetNumGuildMembers(true), 1 do
			local name,rankname,rankindex,level,_,_,_,_,_ = GetGuildRosterInfo(i)
			guildinfo["rank"][rankindex] = rankname
			
			if name ~= playername then
				guildinfo["member"][i] = { ["name"] = name, ["rank"] = rankname, ["level"] = level}
			end
		end
		
		-- re-enable the guild and officer checkboxes (since you are in a guild)
		if frames.AddressFrame then
			frames.AddressFrame.checkboxes.guildcheck:SetDisabled(false)
			frames.AddressFrame.checkboxes.officercheck:SetDisabled(false)
			frames.AddressFrame.officerdrop:SetDisabled(false)
			frames.AddressFrame.maxcheck:SetDisabled(false)
			if not frames.AddressFrame:IsVisible() then
				frames.AddressFrame.officerdrop:SetList(guildinfo["rank"])
			end
		end
	else
		-- disable the guild and officer checkboxes (since you aren't in a guild)
		if frames.AddressFrame then
			frames.AddressFrame.maxcheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.guildcheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.guildcheck:SetValue()
			frames.AddressFrame.checkboxes.officercheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.officercheck:SetValue()
			frames.AddressFrame.officerdrop:SetList()
			frames.AddressFrame.officerdrop:SetDisabled(true)
		end
	end
end

function MM:PLAYER_GUILD_UPDATE()
	-- two different events are called depending on circumstance, so disable checkboxes if this one is
	if not IsInGuild() then
		if frames.AddressFrame then
			frames.AddressFrame.maxcheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.guildcheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.guildcheck:SetValue()
			frames.AddressFrame.checkboxes.officercheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.officercheck:SetValue()
			frames.AddressFrame.officerdrop:SetList()
			frames.AddressFrame.officerdrop:SetDisabled(true)
		end
	end
end

function MM:FRIENDLIST_UPDATE()
	-- index your friends list
	if GetNumFriends() > 0 then
		friends = {}
		for i=1, GetNumFriends(), 1 do
			friends[i],_,_,_,_,_,_,_  = GetFriendInfo(i)
		end
		-- enable the checkbox if you have friends
		if frames.AddressFrame then
			frames.AddressFrame.checkboxes.friendcheck:SetDisabled(false)
		end
	else
		-- disable the checkbox if you don't have friends
		if frames.AddressFrame then
			frames.AddressFrame.checkboxes.friendcheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.friendcheck:SetValue()
		end
	end
end

function MM:ARENA_TEAM_ROSTER_UPDATE()
	-- index any arena teams you are a member of
	if IsInArenaTeam() then
		arenainfo["2v2"] = {}
		arenainfo["3v3"] = {}
		arenainfo["5v5"] = {}
		-- 2v2
		if ArenaTeam_GetTeamSizeID(2) then
			for i=1, GetNumArenaTeamMembers(ArenaTeam_GetTeamSizeID(2)), 1 do
				local name,_,_,_,_,_,_,_,_,_ = GetArenaTeamRosterInfo(ArenaTeam_GetTeamSizeID(2), i)
				if name ~= playername then
					arenainfo["2v2"][i] = name
				end
			end
			-- enable checkbox if in a 2v2
			if frames.AddressFrame then
				frames.AddressFrame.checkboxes.twoscheck:SetDisabled(false)
			end
		else
			-- disable checkbox if you aren't in a 2v2
			if frames.AddressFrame then 
				frames.AddressFrame.checkboxes.twoscheck:SetDisabled(true)
				frames.AddressFrame.checkboxes.twoscheck:SetValue()
			end
		end
		-- repeat for 3v3
		if ArenaTeam_GetTeamSizeID(3) then
			for i=1, GetNumArenaTeamMembers(ArenaTeam_GetTeamSizeID(3)), 1 do
				local name,_,_,_,_,_,_,_,_,_ = GetArenaTeamRosterInfo(ArenaTeam_GetTeamSizeID(3), i)
				if name ~= playername then
					arenainfo["3v3"][i] = name
				end
			end
			if frames.AddressFrame then
				frames.AddressFrame.checkboxes.threescheck:SetDisabled(false)
			end
		else
			if frames.AddressFrame then 
				frames.AddressFrame.checkboxes.threescheck:SetDisabled(true)
				frames.AddressFrame.checkboxes.threescheck:SetValue()
			end
		end
		--repeat for 5v5
		if ArenaTeam_GetTeamSizeID(5) then
			for i=1, GetNumArenaTeamMembers(ArenaTeam_GetTeamSizeID(5)), 1 do
				local name,_,_,_,_,_,_,_,_,_ = GetArenaTeamRosterInfo(ArenaTeam_GetTeamSizeID(5), i)
				if name ~= playername then
					arenainfo["5v5"][i] = name
				end
			end
			if frames.AddressFrame then
				frames.AddressFrame.checkboxes.fivescheck:SetDisabled(false)
			end
		else
			if frames.AddressFrame then 
				frames.AddressFrame.checkboxes.fivescheck:SetDisabled(true)
				frames.AddressFrame.checkboxes.fivescheck:SetValue()
			end
		end
	else
		-- disable checkboxes if you aren't in any arena teams
		if frames.AddressFrame then
			frames.AddressFrame.checkboxes.twoscheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.twoscheck:SetValue()
			frames.AddressFrame.checkboxes.threescheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.threescheck:SetValue()
			frames.AddressFrame.checkboxes.fivescheck:SetDisabled(true)
			frames.AddressFrame.checkboxes.fivescheck:SetValue()			
		end
	end
end

function MM:MAIL_SEND_SUCCESS()
	-- check to see if a mail is currently being sent
	if message.active then
		-- iterate message counters
		message.biter = message.biter - 1
		message.count = message.count + 1
		-- check to see if all the parts of a message have been sent (for the particular player name)
		if message.biter < 1 then
			message.biter = getn(message.body)
			message.niter = message.niter + 1
		end
		-- if there are still more names to send to
		if message.niter <= getn(message.names) then
			message.ready = true
		else
			-- print finished message and errors
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Send Complete."].."  "..L["Total messages sent:"].." "..(tostring(message.count)))
			if getn(message.errors) > 0 then
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["There were errors with the following receipients"].." - "..table.concat(message.errors, ", "))
			end
			message.active = false
		end
	end
end

function MM:MAIL_FAILED()
	-- check to see if a mail is being sent
	if message.active then
		-- increment counters and skip remaining messages if there was an error with a name
		message.biter = getn(message.body)
		message.errors[#message.errors + 1] = message.names[message.niter]
		message.niter = message.niter + 1
		-- if there are more names to send to...
		if message.niter <= getn(message.names) then
			message.ready = true
		else
			-- if done, print finish message
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Send Complete."].."  "..L["Total messages sent:"].." "..(tostring(message.count)))
			if getn(message.errors) > 0 then
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["There were errors with the following receipients"].." - "..table.concat(message.errors, ", "))
			end
			message.active = false
		end
	end
end

--[[--------------------
-- Frame Toggles
--]]--------------------
-- pretty obvious stuff here
function MM:CloseAllFrames()
	for _, frame in pairs(frames) do
		if frame then
			frame:Hide()
		end
	end
end

function MM:ToggleMessageFrame()
	if frames.MessageFrame then
		if frames.MessageFrame:IsVisible() then
			MM:CloseAllFrames()
		else
			if loadedoptions["frameclamp"] then
				frames.MessageFrame.frame:SetClampedToScreen(true)
				frames.MessageFrame.frame:SetFrameLevel(1)
			else
				frames.MessageFrame.frame:SetClampedToScreen(false)
			end
			frames.MessageFrame:Show()
		end
	else
		MM:SpawnMessageFrame()
	end
end

function MM:ToggleAddressFrame()
	if frames.AddressFrame then
		if frames.AddressFrame:IsVisible() then
			frames.AddressFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.AddressFrame.frame:SetClampedToScreen(true)
				frames.AddressFrame.frame:SetFrameLevel(5)
			else
				frames.AddressFrame.frame:SetClampedToScreen(false)
			end
			frames.AddressFrame:Show()
		end
	else
		MM:SpawnAddressFrame()
	end
end

function MM:ToggleGroupFrame()
	if frames.GroupFrame then
		if frames.GroupFrame:IsVisible() then
			frames.GroupFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.GroupFrame.frame:SetClampedToScreen(true)
				frames.GroupFrame.frame:SetFrameLevel(5)
			else
				frames.GroupFrame.frame:SetClampedToScreen(false)
			end
			frames.GroupFrame:Show()
		end
	else
		MM:SpawnGroupFrame()
	end
end

function MM:ToggleCreateFrame()
	if frames.CreateFrame then
		if frames.CreateFrame:IsVisible() then
			frames.CreateFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.CreateFrame.frame:SetClampedToScreen(true)
			else
				frames.CreateFrame.frame:SetClampedToScreen(false)
			end
			frames.CreateFrame:Show()
		end
	else
		MM:SpawnCreateFrame()
	end
end

function MM:ToggleAddFrame()
	if frames.AddFrame then
		if frames.AddFrame:IsVisible() then
			frames.AddFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.AddFrame.frame:SetClampedToScreen(true)
			else
				frames.AddFrame.frame:SetClampedToScreen(false)
			end
			frames.AddFrame:Show()
		end
	else
		MM:SpawnAddFrame()
	end
end

function MM:ToggleEventFrame()
	local _,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,calendarType = CalendarGetEventInfo()
	if frames.EventFrame then
		if frames.EventFrame:IsVisible() then
			frames.EventFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.EventFrame.frame:SetClampedToScreen(true)
				frames.EventFrame.frame:SetFrameLevel(5)
			else
				frames.EventFrame.frame:SetClampedToScreen(false)
			end
			frames.EventFrame:Show()
			if calendarType == "GUILD_EVENT" then
				if frames.EventFrame then
					frames.EventFrame.checkboxes.signedupcheck:SetDisabled(false)
					frames.EventFrame.checkboxes.signedupcheck:SetValue(true)
				end
			else
				if frames.EventFrame then
					frames.EventFrame.checkboxes.signedupcheck:SetDisabled(true)
					frames.EventFrame.checkboxes.signedupcheck:SetValue(false)
				end
			end
		end
	else
		MM:SpawnEventFrame()
		if calendarType == "GUILD_EVENT" then
			if frames.EventFrame then
				frames.EventFrame.checkboxes.signedupcheck:SetDisabled(false)
				frames.EventFrame.checkboxes.signedupcheck:SetValue(true)
			end
		else
			if frames.EventFrame then
				frames.EventFrame.checkboxes.signedupcheck:SetDisabled(true)
				frames.EventFrame.checkboxes.signedupcheck:SetValue(false)
			end
		end
	end
end

function MM:ToggleDraftFrame()
	if frames.DraftFrame then
		if frames.DraftFrame:IsVisible() then
			frames.DraftFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.DraftFrame.frame:SetClampedToScreen(true)
			else
				frames.DraftFrame.frame:SetClampedToScreen(false)
			end
			frames.DraftFrame:Show()
		end
	else
		MM:SpawnDraftFrame()
	end
end

function MM:ToggleSignatureFrame()
	if frames.SignatureFrame then
		if frames.SignatureFrame:IsVisible() then
			frames.SignatureFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.SignatureFrame.frame:SetClampedToScreen(true)
			else
				frames.SignatureFrame.frame:SetClampedToScreen(false)
			end
			frames.SignatureFrame:Show()
		end
	else
		MM:SpawnSignatureFrame()
	end
end

function MM:ToggleCreateSignatureFrame()
	if frames.CreateSignatureFrame then
		if frames.CreateSignatureFrame:IsVisible() then
			frames.CreateSignatureFrame:Hide()
		else
			if loadedoptions["frameclamp"] then
				frames.CreateSignatureFrame.frame:SetClampedToScreen(true)
			else
				frames.CreateSignatureFrame.frame:SetClampedToScreen(false)
			end
			frames.CreateSignatureFrame:Show()
		end
	else
		MM:SpawnCreateSignatureFrame()
	end
end

--[[--------------------
-- Spawn Message Frame
--]]--------------------
function MM:SpawnMessageFrame()
	local MessageFrame = AceGUI:Create("Window")
		MessageFrame:SetTitle(L["MassMailer"])
		MessageFrame:SetWidth(500)
		MessageFrame:SetHeight(610)
		MessageFrame:SetPoint("TOPLEFT", SendMailFrame, "TOPRIGHT", -25, -12)
	
		-- prevent crash from resizing the textbox too small
		MessageFrame.frame:SetMinResize(400, 300)  
		
		if loadedoptions["frameclamp"] then
			MessageFrame.frame:SetClampedToScreen(true)
			MessageFrame.frame:SetFrameLevel(1)
		end
	
	local addressbox = AceGUI:Create("mmMultiLineEditBox")
		addressbox:SetLabel(L["To:"].." "..L["(Comma Separated)"])
		addressbox:HideButton()
		addressbox:SetLength(5000)
		MessageFrame:AddChild(addressbox)
		
	local subjectbox = AceGUI:Create("EditBox")
		subjectbox:SetLabel(L["Subject:"])
		subjectbox:DisableButton(true)
		MessageFrame:AddChild(subjectbox)
	
	local bodybox = AceGUI:Create("mmMultiLineEditBox")
		bodybox:SetLabel(L["Body:"])
		bodybox:SetLength(5000)
		bodybox:HideButton()
		if loadedoptions["insertsig"] then
			bodybox:SetText(loadedsigs[loadedoptions["defaultsig"]])
		end
		MessageFrame:AddChild(bodybox)

	local label = AceGUI:Create("Label")
		label:SetFont("Fonts\\FRIZQT__.TTF", 10, nil)
		label:SetText(" ")
		MessageFrame:AddChild(label)		

	local addressbtn = AceGUI:Create("Button")
		addressbtn:SetText(L["Addresses"])
		MessageFrame:AddChild(addressbtn)
		
	local sigbtn = AceGUI:Create("Button")
		sigbtn:SetText(L["Signature"]) 
		MessageFrame:AddChild(sigbtn)
	
	local draftbtn = AceGUI:Create("Button")
		draftbtn:SetText(L["Drafts"])
		MessageFrame:AddChild(draftbtn)
	
	local sendbtn = AceGUI:Create("Button")
		sendbtn:SetText(L["Send"])
		MessageFrame:AddChild(sendbtn)
		sendbtn:SetDisabled(mailclosed)		
	
	-- callbacks
	MessageFrame:SetCallback("OnClose", function() MM:CloseAllFrames() end)
	addressbtn:SetCallback("OnClick", function() MM:ToggleAddressFrame() end)
	sigbtn:SetCallback("OnClick", function() MM:ToggleSignatureFrame() end)
	draftbtn:SetCallback("OnClick", function() MM:ToggleDraftFrame() end)
	sendbtn:SetCallback("OnClick", function() 
		MM:Send()
	end)
	subjectbox:SetCallback("OnTextChanged", function(widget, event, text) MessageFrame.subject = text end)
	addressbox:SetCallback("OnTextChanged", function (widget, event, text)
		local startpos, endpos,_,_ = text:find("%[.*%]")	
		if startpos then
			local searchstr = string.lower(text:sub(startpos+1, endpos-1))
			local replacestr = ""
			for index, name in pairs (loadedgroups["names"]) do
				if name == searchstr then
					replacestr = table.concat(loadedgroups["members"][index], ", ")
				end
			end
			
			if replacestr:len() <= 0 then
				if searchstr == "friends" then
					replacestr = table.concat(friends, ", ")
				elseif searchstr == "guild" then
					local guild = {}
					for index, item in pairs (guildinfo["member"]) do
						guild[#guild + 1] = item["name"]
					end
					replacestr = table.concat(guild, ", ")
				elseif searchstr == "officers" then
					local officers = {}
					for rank, value in pairs (loadedoptions["officerranks"]) do
						if value then
							for _, member in pairs (guildinfo["member"]) do
								if member["rank"] == guildinfo["rank"][tonumber(rank:sub(-1))] then
									tinsert(officers, member["name"])
								end
							end
						end
					end
					replacestr = table.concat(officers, ", ")
				elseif searchstr == "2v2" or searchstr == "3v3" or searchstr == "5v5" then
					replacestr = table.concat(arenainfo[searchstr], ", ")
				end
			end
			
			if replacestr:len() > 0 then
				addressbox:SetText(text:sub(0, startpos-1)..replacestr..text:sub(endpos+1))
				addressbox:SetCursor(text:sub(0, startpos-1):len() + replacestr:len())
			end
		end
	end)
	bodybox:SetCallback("OnTextChanged", function()
		-- update message frame status text
		if(frames.MessageFrame) then
			label:SetText(L["Characters:"].." "..bodybox:GetText():len()..", "..L["Parts:"].." "..getn(ParseBody(bodybox:GetText())))
		end
	end)
	
	
	-- tooltips
	if loadedoptions["showtooltips"] then
		addressbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Address Book"], L["Open the Address Book"]) end)
		addressbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		sigbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Signature"], L["Open the Signature Editor"]) end)
		sigbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		draftbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Drafts"], L["Open the Draft Editor"]) end)
		draftbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		sendbtn:SetCallback("OnEnter", function()
			local numrecipient = getn(BreakString(addressbox:GetText()))
			local numparts = getn(ParseBody(bodybox:GetText()))
			local cost = numrecipient * numparts * 30
			MM:ShowTooltip(L["Send"], L["Mail the current message"].."\n\n"..L["Number of recipients:"].." "..tostring(numrecipient).."\n"..L["Number of parts:"].." "..tostring(numparts).."\n"..L["Total number of messages:"].." "..tostring(numrecipient * numparts).."\n"..L["Estimated cost:"].." "..floor(abs(cost/10000)).."g"..floor(abs(mod(cost/100,100))).."s"..floor(abs(mod(cost, 100))).."c")
		end)
		sendbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end
	
	-- frame layout
	AceGUI:RegisterLayout("MessageFrame", function(content, children)
		children[1]:SetPoint("TOPLEFT", content, "TOPLEFT", 5, 0)
		children[1]:SetWidth(content:GetWidth())
		children[1]:SetHeight(100)
		children[2]:SetWidth(content:GetWidth())
		children[3]:SetWidth(content:GetWidth())
		children[3]:SetHeight(content:GetHeight() - children[1].frame:GetHeight() - children[2].frame:GetHeight() - children[4].frame:GetHeight() -  children[5].frame:GetHeight())
		children[4]:ClearAllPoints()
		children[4]:SetWidth(content:GetWidth())
		children[4]:SetPoint("TOPLEFT", children[3].frame, "BOTTOMLEFT", 10, -2)
		children[5]:ClearAllPoints()
		children[5]:SetWidth(content:GetWidth()/4)
		children[5]:SetPoint("TOPLEFT", children[4].frame, "BOTTOMLEFT", -10, -5)
		children[6]:ClearAllPoints()
		children[6]:SetWidth(content:GetWidth()/4)
		children[6]:SetPoint("TOPLEFT", children[5].frame, "TOPRIGHT", 0, 0)
		children[7]:SetWidth(content:GetWidth()/4)
		children[7]:SetPoint("TOPLEFT", children[6].frame, "TOPRIGHT", 0, 0)
		children[8]:SetWidth(content:GetWidth()/4)
		children[8]:SetPoint("TOPLEFT", children[7].frame, "TOPRIGHT", 0, 0)
		
	end)
	MessageFrame:SetLayout("MessageFrame")
	
	-- add MessageFrame children	
	MessageFrame.addressbox = addressbox
	MessageFrame.subjectbox = subjectbox
	MessageFrame.bodybox = bodybox
	MessageFrame.label = label
	MessageFrame.sendbtn = sendbtn

	-- add MessageFrame to frames list
	frames.MessageFrame = MessageFrame
end

--[[--------------------
-- Spawn Address Book Frame
--]]--------------------
function MM:SpawnAddressFrame()
	-- function variables
	local checked = {}
	
	-- frames
	local AddressFrame = AceGUI:Create("Window")
		AddressFrame:SetTitle(L["Address Book"])
		AddressFrame:SetWidth(400)
		AddressFrame:SetHeight(250)
		AddressFrame:EnableResize(false)
		AddressFrame:SetPoint("TOPLEFT", frames.MessageFrame.frame, "TOPRIGHT", 0, 0)

		if loadedoptions["frameclamp"] then
			AddressFrame.frame:SetClampedToScreen(true)
			AddressFrame.frame:SetFrameLevel(5)
		end
	local personalgrp = AceGUI:Create("InlineGroup")
		personalgrp:SetTitle(L["Personal"])
		AddressFrame:AddChild(personalgrp)
		
		local friendcheck = AceGUI:Create("CheckBox")
			friendcheck:SetLabel(L["Friends"])
			if GetNumFriends() == 0 then
				friendcheck:SetDisabled(true)
			end
			personalgrp:AddChild(friendcheck)
		
		local guildcheck = AceGUI:Create("CheckBox")
			guildcheck:SetLabel(L["Guild"])
			if not IsInGuild() then
				guildcheck:SetDisabled(true)
			end
			personalgrp:AddChild(guildcheck)
			
		local maxcheck = AceGUI:Create("CheckBox")
			maxcheck:SetLabel(L["80s Only"])
			personalgrp:AddChild(maxcheck)
			
		local officercheck = AceGUI:Create("CheckBox")
			officercheck:SetLabel(L["Officers"])
			if not IsInGuild() then
				officercheck:SetDisabled(true)
			end
			personalgrp:AddChild(officercheck)
			
		local officerdrop = AceGUI:Create("Dropdown")
			officerdrop:SetMultiselect(true)
			if IsInGuild() then
				officerdrop:SetList(guildinfo["rank"])
				for rank,value in pairs(loadedoptions["officerranks"]) do
					if value then
						officerdrop:SetItemValue(tonumber(rank:sub(-1)), true)
						checked[tonumber(rank:sub(-1))] = true
					end
				end
				
			else
				officerdrop:SetDisabled(true)
			end
			personalgrp:AddChild(officerdrop)
		
			-- personal group layout
			AceGUI:RegisterLayout("personalgrp", function(content, children)
				children[1]:SetWidth(content:GetWidth())
				children[2]:SetWidth(content:GetWidth()/3)
				children[3]:SetWidth(content:GetWidth()*2/3)
				children[3]:SetPoint("TOPLEFT", children[2].frame, "TOPRIGHT", 0, 0)
				children[4]:SetWidth(content:GetWidth()/3)
				children[4]:SetPoint("TOPLEFT", children[2].frame, "BOTTOMLEFT", 0, 0)
				children[5]:SetWidth(content:GetWidth()*2/3)
				children[5]:ClearAllPoints()
				children[5]:SetPoint("TOPLEFT", children[4].frame, "TOPRIGHT", 0, 0)
			end)
			personalgrp:SetLayout("personalgrp")
			
	local arenagrp = AceGUI:Create("InlineGroup")
		arenagrp:SetTitle(L["Arena"])
		AddressFrame:AddChild(arenagrp)
		
		local twoscheck = AceGUI:Create("CheckBox")
			twoscheck:SetLabel(L["2v2"])
			if not ArenaTeam_GetTeamSizeID(2) then
				twoscheck:SetDisabled(true)
			end
			arenagrp:AddChild(twoscheck)
			
		local threescheck = AceGUI:Create("CheckBox")
			threescheck:SetLabel(L["3v3"])
			if not ArenaTeam_GetTeamSizeID(3) then
				threescheck:SetDisabled(true)
			end
			arenagrp:AddChild(threescheck)
			
		local fivescheck = AceGUI:Create("CheckBox")
			fivescheck:SetLabel(L["5v5"])
			if not ArenaTeam_GetTeamSizeID(5) then
				fivescheck:SetDisabled(true)
			end
			arenagrp:AddChild(fivescheck)
		
		-- arena group layout
		AceGUI:RegisterLayout("arenagrp", function(content, children)
			for _, child in pairs(children) do
				child:SetWidth(content:GetWidth())
			end
		end)
		arenagrp:SetLayout("arenagrp")
		
	local customgrp = AceGUI:Create("InlineGroup")
		customgrp:SetTitle(L["Custom Groups"])
		AddressFrame:AddChild(customgrp)
		
		local customdrop = AceGUI:Create("Dropdown")
			customdrop:SetLabel(L["Select Group:"])
			customdrop:SetList(loadedgroups["names"])
			customgrp:AddChild(customdrop)
			
		local custombtn = AceGUI:Create("Button")
			custombtn:SetText(L["Edit Groups"])
			customgrp:AddChild(custombtn)
	
		-- custom group layout
		AceGUI:RegisterLayout("customgrp", function(content, children)
			for _, child in pairs(children) do
				child:SetWidth(content:GetWidth()/2 - 5)
			end
			children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 10, -20)
		end)
		customgrp:SetLayout("customgrp")
		
	-- AddressFrame layout
	AceGUI:RegisterLayout("AddressFrame", function(content, children)
		children[1]:SetPoint("TOPLEFT", 5, 0)
		children[1]:SetWidth(content:GetWidth()*2/3)
		children[1]:SetHeight(content:GetHeight()*2/3 - 10)
		children[2]:SetWidth(content:GetWidth()/3)
		children[2]:SetHeight(content:GetHeight()*2/3 - 10)
		children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
		children[3]:SetWidth(content:GetWidth())
		children[3]:SetHeight(content:GetHeight()/3 + 20)
		children[3]:ClearAllPoints()
		children[3]:SetPoint("TOPLEFT", children[1].frame, "BOTTOMLEFT", 0, 0)
	end)
	AddressFrame:SetLayout("AddressFrame")
	
	-- add AddressFrame children
	AddressFrame.checkboxes = {}
	AddressFrame.checkboxes.friendcheck = friendcheck	
	AddressFrame.checkboxes.guildcheck = guildcheck
	AddressFrame.checkboxes.officercheck = officercheck
	AddressFrame.checkboxes.twoscheck = twoscheck
	AddressFrame.checkboxes.threescheck = threescheck
	AddressFrame.checkboxes.fivescheck = fivescheck
	
	AddressFrame.maxcheck = maxcheck
	AddressFrame.officerdrop = officerdrop
	AddressFrame.customdrop = customdrop
	AddressFrame.custombtn = custombtn
	
	-- AddressFrame callbacks
	AddressFrame:SetCallback("OnClose", function()
		if frames.GroupFrame and frames.GroupFrame:IsVisible() then
			MM:ToggleGroupFrame()
		end
	end)
	
	-- personal group callbacks
	friendcheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(AddressFrame.checkboxes, widget, false)
			frames.MessageFrame.addressbox:SetText(table.concat(friends, ", "))
		else
			frames.MessageFrame.addressbox:SetText()
		end
	end)
	maxcheck:SetCallback("OnValueChanged", function(widget, event, value)
		if guildcheck:GetValue() then
			guildcheck:Fire("OnValueChanged", true)
		end
	end)
	
	guildcheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(AddressFrame.checkboxes, widget, false)
			local guild = {}
			for index, item in pairs (guildinfo["member"]) do
				if maxcheck:GetValue() then
					if item["level"] == 80 then
						guild[#guild + 1] = item["name"]
					end
				else
					guild[#guild + 1] = item["name"]
				end
			end
			frames.MessageFrame.addressbox:SetText(table.concat(guild, ", "))
		else
			frames.MessageFrame.addressbox:SetText()
		end
	end)
	
	officerdrop:SetCallback("OnValueChanged", function(widget, event, key, value)
		if value then
			checked[key] = true
		else
			checked[key] = false
		end
		officercheck:SetValue(true)
		officercheck:Fire("OnValueChanged", true)
	end)
	
	officercheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(AddressFrame.checkboxes, widget, false)
			local officers = {}
			for _,member in pairs (guildinfo["member"]) do
				for index, value in pairs(checked) do
					if value then
						if member["rank"] == guildinfo["rank"][index] then
							tinsert(officers, member["name"])
						end
					end
				end
			end
			frames.MessageFrame.addressbox:SetText(table.concat(officers, ", "))
		else
			frames.MessageFrame.addressbox:SetText()
		end
	end)

	-- arena group callbacks
	twoscheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(AddressFrame.checkboxes, widget, false)
			frames.MessageFrame.addressbox:SetText(table.concat(arenainfo["2v2"], ", "))
		else
			frames.MessageFrame.addressbox:SetText()
		end
	end)
	
	threescheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(AddressFrame.checkboxes, widget, false)
			frames.MessageFrame.addressbox:SetText(table.concat(arenainfo["3v3"], ", "))
		else
			frames.MessageFrame.addressbox:SetText()
		end
	end)
	
	fivescheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(AddressFrame.checkboxes, widget, false)
			frames.MessageFrame.addressbox:SetText(table.concat(arenainfo["5v5"], ", "))
		else
			frames.MessageFrame.addressbox:SetText()
		end
	end)
	
	-- custom group callbacks
	custombtn:SetCallback("OnClick", function() MM:ToggleGroupFrame() end)
	customdrop:SetCallback("OnValueChanged", function(widget, event, key)
		CheckOthers(AddressFrame.checkboxes, nil, false)
		frames.MessageFrame.addressbox:SetText(table.concat(loadedgroups["members"][key], ", "))
		customdrop:SetValue()
	end)
	
	-- tooltips
	if loadedoptions["showtooltips"] then
		friendcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Friends"], L["Insert your friends list into the address box."].."\n"..L["Will overwrite any current addresses."]) end)
		friendcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		guildcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Guild"], L["Insert the members of your guild into the address box."].."\n"..L["Will overwrite any current addresses."]) end)
		guildcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		maxcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["80s Only"], L["Restrict guild members to Level 80 characters only."]) end)
		maxcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		officercheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Officers"], L["Insert the selected officer ranks into address box."].."\n"..L["Will overwrite any current addresses."]) end)
		officercheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		officerdrop:SetCallback("OnEnter", function() MM:ShowTooltip(L["Rank"], L["Select which officer ranks to insert."]) end)
		officerdrop:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		twoscheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["2v2"], L["Insert the members of your 2v2 team into the address box."].."\n"..L["Will overwrite any current addresses."]) end)
		twoscheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		threescheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["3v3"], L["Insert the members of your 3v3 team into the address box."].."\n"..L["Will overwrite any current addresses."]) end)
		threescheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		fivescheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["5v5"], L["Insert the members of your 5v5 team into the address box."].."\n"..L["Will overwrite any current addresses."]) end)
		fivescheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		customdrop:SetCallback("OnEnter", function() MM:ShowTooltip(L["Select Group:"]:sub(0, -2), L["Insert a custom group into the address box."].."\n"..L["Will overwrite any current addresses."]) end)
		customdrop:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		custombtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Edit Groups"], L["Open the group editor."]) end)
		custombtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end
	
	-- add AddressFrame to frames list
	frames.AddressFrame = AddressFrame
end		

--[[--------------------
-- Spawn Group Editor Frame
--]]--------------------
function MM:SpawnGroupFrame()
	local GroupFrame = AceGUI:Create("Window")
		GroupFrame:SetTitle(L["Group Editor"])
		GroupFrame:SetWidth(400)
		GroupFrame:SetHeight(410)
		GroupFrame:EnableResize(false)
		GroupFrame:SetPoint("TOPLEFT", frames.AddressFrame.frame, "BOTTOMLEFT", 0, 0)
		
		if loadedoptions["frameclamp"] then
			GroupFrame.frame:SetClampedToScreen(true)
			GroupFrame.frame:SetFrameLevel(5)
		end
		
		-- add GroupFrame variables
		GroupFrame.currentgroup = nil
		GroupFrame.currentcontact = nil
		
		local groupms = AceGUI:Create("MultiSelect")
			groupms:SetHeight(250)
			groupms:SetRelativeWidth(0.5)
			groupms:SetMultiSelect(false)
			groupms:SetLabel(L["Groups:"])
			groupms:SetItemList(loadedgroups["names"])
			GroupFrame:AddChild(groupms)
		
		local contactms = AceGUI:Create("MultiSelect")
			contactms:SetHeight(250)
			contactms:SetRelativeWidth(0.5)
			contactms:SetMultiSelect(true)
			contactms:SetLabel(L["Group Contacts:"])
			GroupFrame:AddChild(contactms)
		
		local controlsgrp = AceGUI:Create("InlineGroup")
			controlsgrp:SetFullWidth(true)
			controlsgrp:SetTitle("")
			GroupFrame:AddChild(controlsgrp)
			
			local groupbox = AceGUI:Create("EditBox")
				groupbox:SetLabel(L["Group Name:"])
				controlsgrp:AddChild(groupbox)
			
			local groupbtn = AceGUI:Create("Button")
				groupbtn:SetText(L["New Group"])
				controlsgrp:AddChild(groupbtn)
			
			local groupdeletebtn = AceGUI:Create("Button")
				groupdeletebtn:SetText(L["Delete"])
				controlsgrp:AddChild(groupdeletebtn)
			
			local contactbox = AceGUI:Create("EditBox")
				contactbox:SetLabel(L["Contact Name:"])
				controlsgrp:AddChild(contactbox)
			
			local contactbtn = AceGUI:Create("Button")
				contactbtn:SetText(L["New Contact"])
				controlsgrp:AddChild(contactbtn)
			
			local contactdeletebtn = AceGUI:Create("Button")
				contactdeletebtn:SetText(L["Delete"])
				controlsgrp:AddChild(contactdeletebtn)
				
		-- controls group layout
		AceGUI:RegisterLayout("controlsgrp", function(content, children)
			children[1]:SetWidth(content:GetWidth()/3 - 5)
			children[2]:SetWidth(content:GetWidth()/3 + 10)
			children[3]:SetWidth(content:GetWidth()/3 - 20)
			children[4]:SetWidth(content:GetWidth()/3 - 5)
			children[5]:SetWidth(content:GetWidth()/3 + 10)
			children[6]:SetWidth(content:GetWidth()/3 - 20)
			children[1]:SetPoint("TOPLEFT", 5, 0)
			children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 10, -18)
			children[3]:SetPoint("TOPLEFT", children[2].frame, "TOPRIGHT", 5, 0)
			children[4]:SetPoint("TOPLEFT", children[1].frame, "BOTTOMLEFT", 0, 0)
			children[5]:SetPoint("TOPLEFT", children[4].frame, "TOPRIGHT", 10, -18)
			children[6]:SetPoint("TOPLEFT", children[5].frame, "TOPRIGHT", 5, 0)
		end)
		controlsgrp:SetLayout("controlsgrp")
		
	-- GroupFrame callbacks	
	GroupFrame:SetCallback("OnClose", function()
		groupms:SetSelected(nil, nil)
		contactms:SetItemList()
		groupbox:SetText()
		contactbox:SetText()
		GroupFrame.currentgroup = nil
		GroupFrame.currentcontact = nil
		if frames.CreateFrame then
			frames.CreateFrame:Hide()
		end
		if frames.AddFrame then
			frames.AddFrame:Hide()
		end
	end)
	
	groupms:SetCallback("OnLabelClick", function(widget, event, item)
		if groupms:IsSelected(item) then
			local key = nil
			for index, value in pairs (loadedgroups["names"]) do
				if groupms:GetText(item) == value then
					key = index
				end
			end
			GroupFrame.currentgroup = loadedgroups["names"][key]
			contactms:SetItemList(loadedgroups["members"][key])
			groupbox:SetText(groupms:GetText(item))
			contactbox:SetText()
			GroupFrame.currentcontact = nil
		else
			GroupFrame.currentgroup = nil
			contactms:SetItemList()
			groupbox:SetText()
			contactbox:SetText()
		end
	end)
	
	contactms:SetCallback("OnLabelClick", function(widget, event, item)
		local selected = contactms:GetSelected()
		if getn(selected) > 1 then
			contactbox:SetText("<"..L["Multiple"]..">")
			GroupFrame.currentcontact = "multiple"
		elseif getn(selected) == 1 then
			contactbox:SetText(contactms:GetText(selected[1]))
			GroupFrame.currentcontact = contactms:GetText(selected[1])
		else
			contactbox:SetText()
			GroupFrame.currentcontact = nil
		end
	end)
	
	-- controls group callbacks
	groupbtn:SetCallback("OnClick", function() MM:ToggleCreateFrame() end)
	contactbtn:SetCallback("OnClick", function()
		if GroupFrame.currentgroup then
			MM:ToggleAddFrame()
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a group to edit."])
		end
	end)
	groupdeletebtn:SetCallback("OnClick", function()
		if GroupFrame.currentgroup then
			for index, value in pairs (loadedgroups["names"]) do
				if value == GroupFrame.currentgroup then
					tremove(loadedgroups["names"], index)
					tremove(loadedgroups["members"], index)
					self.db.profile.groups[GroupFrame.currentgroup] = nil
				end
			end
			groupms:SetItemList(loadedgroups["names"])
			contactms:SetItemList()
			groupbox:SetText()
			contactbox:SetText()
			GroupFrame.currentgroup = nil
			frames.AddressFrame.customdrop:SetList(loadedgroups["names"])
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a group to delete."])
		end
	end)
	contactdeletebtn:SetCallback("OnClick", function()
		if GroupFrame.currentcontact then
			local key = nil
			for index, value in pairs (loadedgroups["names"]) do
				if value == GroupFrame.currentgroup then
					key = index
				end
			end
			local selected = contactms:GetSelected()

			for _, value in pairs (selected) do
				for index, item in pairs (loadedgroups["members"][key]) do
					if item == contactms:GetText(value) then
						tremove(loadedgroups["members"][key], index)
					end
				end
			end
			self.db.profile.groups[GroupFrame.currentgroup] = loadedgroups["members"][key]
			contactms:SetItemList(loadedgroups["members"][key])
			GroupFrame.currentcontact = nil
			contactbox:SetText()
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a contact to delete."])
		end
	end)
	groupbox:SetCallback("OnEnterPressed", function(widget, event, text)
		if GroupFrame.currentgroup then
			local found = false
			for _, value in pairs(loadedgroups["names"]) do
				if value == text then
					found = true
				end
			end
			
			if not found then
				local key = nil
				for index, value in pairs (loadedgroups["names"]) do
					if value == GroupFrame.currentgroup then
						loadedgroups["names"][index] = text
						key = index
					end
				end
				self.db.profile.groups = {}
				for index, value in pairs(loadedgroups["names"]) do
					self.db.profile.groups[value] = loadedgroups["members"][index]
				end
				GroupFrame.currentgroup = text
				groupms:SetItemList(loadedgroups["names"])
				groupms:SetSelected(groupms:GetItem(loadedgroups["names"][key]), true)
				contactms:SetItemList(loadedgroups["members"][key])
				frames.AddressFrame.customdrop:SetList(loadedgroups["names"])
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Group name already exists."])
			end
		
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a group to edit."])
		end
	end)
	contactbox:SetCallback("OnEnterPressed", function(widget, event, text)
		if GroupFrame.currentcontact then
			if GroupFrame.currentcontact ~= "multiple" then
				local key = nil
				for index, value in pairs (loadedgroups["names"]) do
					if value == GroupFrame.currentgroup then
						key = index
					end
				end
				local found = false
				for _, value in pairs(loadedgroups["members"][key]) do
					if value == text then
						found = true
					end
				end
				if not found then 
					for index, value in pairs (loadedgroups["members"][key]) do
						if value == GroupFrame.currentcontact then 
							loadedgroups["members"][key][index] = text
							GroupFrame.currentcontact = text
						end
					end
					contactms:SetItemList(loadedgroups["members"][key])
					self.db.profile.groups[GroupFrame.currentgroup] = loadedgroups["members"][key]
					contactms:SetSelected(contactms:GetItem(text), true)
				else
					print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Contact already exists in group"])
				end
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Multiple contacts selected.  Please choose a single contact to edit."])
			end
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a contact to edit."])
		end
	end)
	
	-- tooltips
	if loadedoptions["showtooltips"] then
		groupbox:SetCallback("OnEnter", function() MM:ShowTooltip(L["Group Name:"]:sub(0, -2), L["Edit the name of the selected group."]) end)
		groupbox:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		groupbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["New Group"], L["Create a new group."]) end)
		groupbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		groupdeletebtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Delete"], L["Delete the selected group."]) end)
		groupdeletebtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		contactbox:SetCallback("OnEnter", function() MM:ShowTooltip(L["Contact Name:"]:sub(0, -2), L["Edit the name of the selected contact."]) end)
		contactbox:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		contactbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["New Contact"], L["Create a new contact in the selected group."]) end)
		contactbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		contactdeletebtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Delete"], L["Delete the selected contact(s)."]) end)
		contactdeletebtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end
	
	-- GroupFrame layout
	AceGUI:RegisterLayout("GroupFrame", function(content, children)
		children[1]:SetPoint("TOPLEFT", 5, -5)
		children[1]:SetHeight(content:GetHeight()*2/3)
		children[2]:SetHeight(content:GetHeight()*2/3)
		children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
		children[3]:SetHeight(content:GetHeight()/3+10)
		children[3]:SetPoint("TOPLEFT", children[1].frame, "BOTTOMLEFT", 0, 5)
	end)
	GroupFrame:SetLayout("GroupFrame")
	
	-- add GroupFrame children
	GroupFrame.groupms = groupms
	GroupFrame.contactms = contactms
	GroupFrame.groupbox = groupbox
	GroupFrame.groupbtn = groupbtn
	GroupFrame.groupdeletebtn = groupdeletebtn
	GroupFrame.contactbox = contactbox
	GroupFrame.contactbtn = contactbtn
	GroupFrame.contactdeletebtn = contactdeletebtn
	
	-- add GroupFrame to frames list
	frames.GroupFrame = GroupFrame	
end

--[[--------------------
-- Spawn Create Group Frame
--]]--------------------
function MM:SpawnCreateFrame()
	-- function variables
	local grouptext = nil
	
	-- frames
	local CreateFrame = AceGUI:Create("Window")
		CreateFrame:SetWidth(220)
		CreateFrame:SetHeight(120)
		CreateFrame:SetTitle(L["New Group"])
		CreateFrame:EnableResize(false)
		
		CreateFrame.frame:SetFrameLevel(50)
		
		if loadedoptions["frameclamp"] then
			CreateFrame.frame:SetClampedToScreen(true)
		end

		local createbox = AceGUI:Create("EditBox")
			createbox:DisableButton(true)
			createbox:SetLabel(L["Group Name:"])
			createbox:SetFullWidth(true)
			CreateFrame:AddChild(createbox)
			
		local createbtn = AceGUI:Create("Button")
			createbtn:SetText(L["Create"])
			createbtn:SetFullWidth(true)
			CreateFrame:AddChild(createbtn)
	
	-- CreateFrame callbacks
	createbox:SetCallback("OnTextChanged", function(widget, event, text) grouptext = text end)
	createbox:SetCallback("OnEnterPressed", function() createbtn:Fire("OnClick") end)
	createbtn:SetCallback("OnClick", function()
		if grouptext then
			local found = false
			for _, group in pairs (loadedgroups["names"]) do
				if group == grouptext then
					found = true
				end
			end
			
			if not found then
				tinsert(loadedgroups["names"], grouptext)
				loadedgroups["members"][#loadedgroups["members"] + 1] = {}
				self.db.profile.groups[grouptext] = {}
				frames.GroupFrame.groupms:SetItemList(loadedgroups["names"])
				frames.GroupFrame.groupms:SetSelected(frames.GroupFrame.groupms:GetItem(grouptext), true)
				frames.GroupFrame.groupbox:SetText(grouptext)
				frames.GroupFrame.currentgroup = grouptext
				frames.GroupFrame.contactms:SetItemList()
				frames.GroupFrame.contactbox:SetText()
				frames.AddressFrame.customdrop:SetList(loadedgroups["names"])
				createbox:SetText()
				grouptext = nil
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Group name already exists."])
			end
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please enter a group name."])
		end
	end)
	
	if loadedoptions["showtooltips"] then
		createbox:SetCallback("OnEnter", function() MM:ShowTooltip(L["Group Name:"]:sub(0, -2), L["Enter the new group name."]) end)
		createbox:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		createbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Create"], L["Creates the new group."]) end)
		createbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end

	-- add GroupFrame children
	CreateFrame.createbox = createbox
	CreateFrame.createbtn = createbtn
	
	-- add CreateFrame to frames list
	frames.CreateFrame = CreateFrame
end

--[[--------------------
-- Spawn Add Contact Frame
--]]--------------------
function MM:SpawnAddFrame()
	-- function variables
	local contacttext = nil
	
	-- frames
	local AddFrame = AceGUI:Create("Window")
		AddFrame:SetWidth(220)
		AddFrame:SetHeight(120)
		AddFrame:SetTitle(L["New Contact"])
		AddFrame:EnableResize(false)
		
		AddFrame.frame:SetFrameLevel(50)

		if loadedoptions["frameclamp"] then
			AddFrame.frame:SetClampedToScreen(true)
		end

		local addbox = AceGUI:Create("EditBox")
			addbox:DisableButton(true)
			addbox:SetLabel(L["Contact Name:"])
			addbox:SetFullWidth(true)
			AddFrame:AddChild(addbox)
			
		local addbtn = AceGUI:Create("Button")
			addbtn:SetText(L["Create"])
			addbtn:SetFullWidth(true)
			AddFrame:AddChild(addbtn)
	
	-- CreateFrame callbacks
	addbox:SetCallback("OnTextChanged", function(widget, event, text) contacttext = text end)
	addbox:SetCallback("OnEnterPressed", function() addbtn:Fire("OnClick") end)
	addbtn:SetCallback("OnClick", function()
		if frames.GroupFrame.currentgroup then
			if contacttext then
				local key = nil
				for index, value in pairs(loadedgroups["names"]) do
					if value == frames.GroupFrame.currentgroup then
						key = index
					end
				end
				
				local found = false
				for _, item in pairs(loadedgroups["members"][key]) do
					if item == contacttext then
						found = true
					end
				end
				
				if not found then 
					tinsert(loadedgroups["members"][key], contacttext)
					self.db.profile.groups[frames.GroupFrame.currentgroup] = loadedgroups["members"][key]
					frames.GroupFrame.contactms:SetItemList(loadedgroups["members"][key])
					frames.GroupFrame.contactbox:SetText()
					frames.GroupFrame.currentcontact = nil
					addbox:SetText()
					contacttext = nil
					
				else
					print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Contact already exists in group"].." - "..frames.GroupFrame.currentgroup)
				end
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please enter a contact name."])
			end
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a group to edit."])
		end
	end)
	
	-- tooltips
	if loadedoptions["showtooltips"] then
		addbox:SetCallback("OnEnter", function() MM:ShowTooltip(L["Contact Name:"]:sub(0, -2), L["Enter the new contact name."]) end)
		addbox:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		addbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Create"], L["Creates the new contact in the selected group."]) end)
		addbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end
	
	-- add GroupFrame children
	AddFrame.addbox = addbox
	AddFrame.addbtn = addbtn
	
	-- add CreateFrame to frames list
	frames.AddFrame = AddFrame
end

--[[--------------------
-- Spawn Draft Frame
--]]--------------------
function MM:SpawnDraftFrame()
	-- function variables
	local draftlist = {}
	
	-- frames
	local DraftFrame = AceGUI:Create("Window")
		DraftFrame:SetTitle(L["Drafts"])
		DraftFrame:SetWidth(600)
		DraftFrame:SetHeight(360)
		DraftFrame:EnableResize(false)
		
		DraftFrame.frame:SetFrameLevel(20)
		
		if loadedoptions["frameclamp"] then
			DraftFrame.frame:SetClampedToScreen(true)
		end
		
	local draftms = AceGUI:Create("MultiSelect")
		draftms:SetHeight(250)
		draftms:SetMultiSelect(false)
		draftms:SetLabel(L["Drafts"]..":")
		for title,_ in pairs(loadeddrafts) do
			draftlist[#draftlist + 1] = title
		end
		draftms:SetItemList(draftlist)
		DraftFrame:AddChild(draftms)
	
	local messagebox = AceGUI:Create("MessageFrame")
		messagebox:SetHeight(250)
		messagebox:SetLabel(L["Message"]..":")
		DraftFrame:AddChild(messagebox)
		
	local optiongrp = AceGUI:Create("InlineGroup")
		optiongrp:SetTitle("")
		DraftFrame:AddChild(optiongrp)
		
		local loadbtn = AceGUI:Create("Button")
			loadbtn:SetText(L["Load"])
			optiongrp:AddChild(loadbtn)
		
		local savebtn = AceGUI:Create("Button")
			savebtn:SetText(L["Save"])
			optiongrp:AddChild(savebtn)
			
		local deletebtn = AceGUI:Create("Button")
			deletebtn:SetText(L["Delete"])
			optiongrp:AddChild(deletebtn)
			
		AceGUI:RegisterLayout("OptionGroup", function(content, children)
			for _, child in pairs(children) do
				child:SetWidth(content:GetWidth()/3)
			end
			children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
			children[3]:SetPoint("TOPLEFT", children[2].frame, "TOPRIGHT", 0, 0)
		end)
		optiongrp:SetLayout("OptionGroup")
		
	-- frame layout
	AceGUI:RegisterLayout("DraftFrame", function(content, children)
		children[1]:SetPoint("TOPLEFT", content, "TOPLEFT", 5, 0)
		children[1]:SetWidth(content:GetWidth()/3)
		children[2]:SetWidth(content:GetWidth()*2/3)
		children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
		children[3]:SetWidth(content:GetWidth())
		children[3]:SetPoint("TOPLEFT", children[1].frame, "BOTTOMLEFT", 0, 0)
		children[3]:SetHeight(65)
	end)
	DraftFrame:SetLayout("DraftFrame")	
	
	-- callbacks
	savebtn:SetCallback("OnClick", function()
		local draft = {
			["names"] = frames.MessageFrame.addressbox:GetText(),
			["subject"] = frames.MessageFrame.subject,
			["body"] = frames.MessageFrame.bodybox:GetText(),
			["date"] = date(),
		}
		
		if not draft["subject"] then
			draft["subject"] = L["Untitled"]
		end
		
		if (loadeddrafts[draft["subject"].." - "..date("%x")]) then
			local found = false
			local iter = 1
			while (found == false) do
				if loadeddrafts[draft["subject"].." #"..tostring(iter).." - "..date("%x")] then
					iter = iter + 1
				else
					self.db.profile.drafts[draft["subject"].." #"..tostring(iter).." - "..date("%x")] = draft
					loadeddrafts[draft["subject"].." #"..tostring(iter).." - "..date("%x")] = draft
					found = true
				end
			end
		else
			self.db.profile.drafts[draft["subject"].." - "..date("%x")] = draft
			loadeddrafts[draft["subject"].." - "..date("%x")] = draft
		end
		
		draftlist = {}
		
		for title,_ in pairs (loadeddrafts) do
			draftlist[#draftlist + 1] = title
		end
		
		draftms:SetItemList(draftlist)
	end)
	deletebtn:SetCallback("OnClick", function()
		local key = nil
		for index,item in pairs (draftlist) do
			if draftms:IsSelected(draftms:GetItem(item)) then
				key = index
			end
		end
		
		if key then
			self.db.profile.drafts[draftlist[key]] = nil
			loadeddrafts[draftlist[key]] = nil
			tremove(draftlist, key)
			draftms:SetItemList(draftlist)
			messagebox:SetText()
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a draft to delete."])
		end
		
	end)
	loadbtn:SetCallback("OnClick", function()
		local selected = draftms:GetSelected()
		
		if getn(selected) >= 1 then
			local current = loadeddrafts[draftms:GetText(selected[1])]
			frames.MessageFrame.addressbox:SetText(current["names"])
			if current["subject"] == L["Untitled"] then
				frames.MessageFrame.subjectbox:SetText()
				frames.MessageFrame.subject = nil --test
			else
				frames.MessageFrame.subjectbox:SetText(current["subject"])
				frames.MessageFrame.subject = current["subject"] --test
			end
			frames.MessageFrame.bodybox:SetText(current["body"])
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please selected a draft to load."])
		end
		
	end)
	draftms:SetCallback("OnLabelClick", function (widget, event, value)
		if draftms:IsSelected(this) then	
			local key = nil
			for index, value in pairs (draftlist) do
				if draftms:GetText(this) == value then
					key = index
				end
			end
			
			local str = L["Subject:"].."  "..loadeddrafts[draftlist[key]]["subject"].."\n\n"..L["Date"]..":  "..loadeddrafts[draftlist[key]]["date"].."\n\n"..L["To:"].."  "..loadeddrafts[draftlist[key]]["names"].."\n\n"..L["Body:"].."\n\n"..loadeddrafts[draftlist[key]]["body"]
			messagebox:SetText(str)
		else
			messagebox:SetText("")
		end
	end)
	DraftFrame:SetCallback("OnClose", function()
		draftms:SetSelected()
		messagebox:SetText("")
	end)
	
	-- tooltips
	if loadedoptions["showtooltips"] then
		loadbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Load"], L["Load the selected draft into the message frame."].."\n"..L["Will overwrite any current message."]) end)
		loadbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		savebtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Save"], L["Save the current message as a draft."]) end)
		savebtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		deletebtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Delete"], L["Delete the selected draft."]) end)
		deletebtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end
	
	-- add DraftFrame children
	DraftFrame.draftlist = draftlist

	-- add DraftFrame to frames list
	frames.DraftFrame = DraftFrame
end

--[[--------------------
-- Spawn Signature Frame
--]]--------------------
function MM:SpawnSignatureFrame()
	-- function variables
	local currentsig = nil
	if loadedoptions["insertsig"] then
		currentsig = loadedsigs[loadedoptions["defaultsig"]]
	end

	-- frames
	local SignatureFrame = AceGUI:Create("Window")
		SignatureFrame:SetTitle(L["Signature"])
		SignatureFrame:SetWidth(600)
		SignatureFrame:SetHeight(400)
		SignatureFrame:EnableResize(false)
		
		SignatureFrame.frame:SetFrameLevel(20)

		local sigms = AceGUI:Create("MultiSelect")
			local siglist = {}
			for title,_ in pairs (loadedsigs) do
				siglist[#siglist + 1] = title
			end
			sigms:SetItemList(siglist)
			sigms:SetMultiSelect(false)
			SignatureFrame:AddChild(sigms)
			
		local titlebox = AceGUI:Create("EditBox")
			titlebox:SetLabel(L["Signature Title:"])
			SignatureFrame:AddChild(titlebox)
		
		local createbtn = AceGUI:Create("Button")
			createbtn:SetText(L["New Signature"])
			SignatureFrame:AddChild(createbtn)
		
		local sigbox = AceGUI:Create("mmMultiLineEditBox")
			sigbox:HideButton()
			sigbox:SetLabel(L["Signature"]..":")
			SignatureFrame:AddChild(sigbox)	
			
		local controlsgrp = AceGUI:Create("InlineGroup")
			controlsgrp:SetTitle("")
			SignatureFrame:AddChild(controlsgrp)

			local deletebtn = AceGUI:Create("Button")
				deletebtn:SetText(L["Delete"])
				controlsgrp:AddChild(deletebtn)
				
			local insertbtn = AceGUI:Create("Button")
				insertbtn:SetText(L["Insert"])
				controlsgrp:AddChild(insertbtn)
				
		-- controlsgrp layout
		AceGUI:RegisterLayout("SigControls", function(content, children)
			for _,child in pairs(children) do
				child:SetWidth(content:GetWidth()/2)
			end
			children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
		end)
		controlsgrp:SetLayout("SigControls")
		
		-- SignatureFrame layout
		AceGUI:RegisterLayout("SignatureFrame", function(content, children)
			children[1]:SetPoint("TOPLEFT", content, 5,0)
			children[1]:SetWidth(content:GetWidth()/3)
			children[1]:SetHeight(content:GetHeight()-45)
			children[2]:SetWidth(content:GetWidth()/3-5)
			children[2]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
			children[3]:SetWidth(content:GetWidth()/3-5)
			children[3]:SetPoint("TOPLEFT", children[2].frame, "TOPRIGHT", 10, -18)
			children[4]:SetWidth(content:GetWidth()*2/3)
			children[4]:SetHeight(content:GetHeight() - children[2].frame:GetHeight()-43)
			children[4]:SetPoint("TOPLEFT", children[2].frame, "BOTTOMLEFT", 0, 0)
			children[5]:SetWidth(content:GetWidth())
			children[5]:SetPoint("TOPLEFT", children[1].frame, "BOTTOMLEFT", 0, 10)
			children[5]:SetHeight(65)
		end)
		SignatureFrame:SetLayout("SignatureFrame")

		-- SignatureFrame callbacks
		sigms:SetCallback("OnLabelClick", function()
			if sigms:IsSelected(this) then
				SignatureFrame.selectedsig = sigms:GetText(this)
				titlebox:SetText(sigms:GetText(this))
				sigbox:SetText(loadedsigs[sigms:GetText(this)])
			else
				titlebox:SetText()
				sigbox:SetText()
				SignatureFrame.selectedsig = nil
			end
		end)
		titlebox:SetCallback("OnEnterPressed", function(widget, event, text)
			if SignatureFrame.selectedsig then
				if text and text:len() > 0 then
					if not loadedsigs[text] then
						local oldsig = loadedsigs[SignatureFrame.selectedsig]
						self.db.profile.signatures[SignatureFrame.selectedsig] = nil
						loadedsigs[SignatureFrame.selectedsig] = nil
						self.db.profile.signatures[text] = oldsig
						loadedsigs[text] = oldsig
						
						local list = {}
						for title,_ in pairs (loadedsigs) do
							list[#list + 1] = title
						end
						sigms:SetItemList(list)
						
						sigms:SetSelected(sigms:GetItem(text), true)
						SignatureFrame.selectedsig = text
					else
						print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["A signature with that title already exists.  Please choose another title."])
					end
				else
					print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please enter a signature title."])
				end
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a signature to edit."])
			end
		end)
		sigbox:SetCallback("OnTextChanged", function(widget, event, text)
			if SignatureFrame.selectedsig then
				self.db.profile.signatures[SignatureFrame.selectedsig] = text
				loadedsigs[SignatureFrame.selectedsig] = text
			end
				
		end)
		createbtn:SetCallback("OnClick", function() MM:ToggleCreateSignatureFrame() end)
		deletebtn:SetCallback("OnClick", function()
			if SignatureFrame.selectedsig then
				if SignatureFrame.selectedsig == loadedoptions["defaultsig"] then
					print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["You have deleted the default signature.  Please set a new default signature in the MassMailer options."])
				end
				
				self.db.profile.signatures[SignatureFrame.selectedsig] = nil
				loadedsigs[SignatureFrame.selectedsig] = nil
				
				local list = {}
				for title,_ in pairs (loadedsigs) do
					list[#list + 1] = title
				end
				titlebox:SetText()
				sigbox:SetText()
				SignatureFrame.selectedsig = nil
				sigms:SetItemList(list)
					
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a signature to delete."])
			end
		end)
		insertbtn:SetCallback("OnClick", function()
			
			if SignatureFrame.selectedsig then
				if currentsig and currentsig:len() > 0 then
					local start,_,_,_ = string.find(frames.MessageFrame.bodybox:GetText(), currentsig, -currentsig:len())
					if start then
						frames.MessageFrame.bodybox:SetText(frames.MessageFrame.bodybox:GetText():sub(0, start-1)..sigbox:GetText())
					else
						frames.MessageFrame.bodybox:SetText(frames.MessageFrame.bodybox:GetText().."\n"..sigbox:GetText())
					end
				else
					frames.MessageFrame.bodybox:SetText(frames.MessageFrame.bodybox:GetText().."\n"..sigbox:GetText())
				end
				currentsig = loadedsigs[SignatureFrame.selectedsig]
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please select a signature to insert."])
			end
		end)

	-- tooltips
	if loadedoptions["showtooltips"] then
		titlebox:SetCallback("OnEnter", function() MM:ShowTooltip(L["Signature Title:"]:sub(0, -2), L["Edit the selected signature's title."]) end)
		titlebox:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		createbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["New Signature"], L["Create a new signature."]) end)
		createbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		deletebtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Delete"], L["Delete the selected signature."]) end)
		deletebtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		insertbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Insert"], L["Insert the selected signature into the current message."]) end)
		insertbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end
	-- add SignatureFrame children
	SignatureFrame.titlebox = titlebox
	SignatureFrame.sigbox = sigbox
	SignatureFrame.sigms = sigms

	-- add SignatureFrame to frames list
	frames.SignatureFrame = SignatureFrame
	
end

--[[--------------------
-- Spawn Create Signature Frame
--]]--------------------
function MM:SpawnCreateSignatureFrame()
	-- function variables
	local currenttitle = nil
	
	-- frames
	local CreateSignatureFrame = AceGUI:Create("Window")
		CreateSignatureFrame:SetWidth(220)
		CreateSignatureFrame:SetHeight(120)
		CreateSignatureFrame:SetTitle(L["New Signature"])
		CreateSignatureFrame:EnableResize(false)
		
		CreateSignatureFrame.frame:SetFrameLevel(50)

		if loadedoptions["frameclamp"] then
			CreateSignatureFrame.frame:SetClampedToScreen(true)
		end

		local addbox = AceGUI:Create("EditBox")
			addbox:DisableButton(true)
			addbox:SetLabel(L["Signature Title:"])
			addbox:SetFullWidth(true)
			CreateSignatureFrame:AddChild(addbox)
			
		local addbtn = AceGUI:Create("Button")
			addbtn:SetText(L["Create"])
			addbtn:SetFullWidth(true)
			CreateSignatureFrame:AddChild(addbtn)
			
	-- CreateSignatureFrame callbacks
	addbox:SetCallback("OnTextChanged", function(widget, event, text) currenttitle = text end)
	addbox:SetCallback("OnEnterPressed", function(widget, event, text)
		if text and text:len() > 0 then
			local found = nil
			for title,_ in pairs (loadedsigs) do
				if text == title then
					found = true
				end
			end
			
			if not found then
				self.db.profile.signatures[text] = ""
				loadedsigs[text] = ""
				
				local list = {}
				for t,_ in pairs (loadedsigs) do
					list[#list + 1] = t
				end
				addbox:SetText()
				currenttitle = nil
				frames.SignatureFrame.sigms:SetItemList(list)
				frames.SignatureFrame.sigms:SetSelected(frames.SignatureFrame.sigms:GetItem(text), true)
				frames.SignatureFrame.titlebox:SetText(text)
				frames.SignatureFrame.sigbox:SetText()
				frames.SignatureFrame.selectedsig = text
				
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["A signature with that title already exists.  Please choose another title."])
			end
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please enter a signature title."])
		end
	end)
	addbtn:SetCallback("OnClick", function() addbox:Fire("OnEnterPressed", currenttitle) end)
			
	-- add CreateSignatureFrame to frames list
	frames.CreateSignatureFrame = CreateSignatureFrame
end

--[[--------------------
-- Spawn Event Frame
--]]--------------------
function MM:SpawnEventFrame()
	local EventFrame = AceGUI:Create("Window")
		EventFrame:SetTitle(L["Event Mailer"]) 
		EventFrame:SetWidth(320)
		EventFrame:SetHeight(280)
		EventFrame:SetPoint("TOPLEFT", CalendarFrame, "TOPRIGHT", 340, -24)
		EventFrame:EnableResize(false)
		
		if loadedoptions["frameclamp"] then
			EventFrame.frame:SetClampedToScreen(true)
			EventFrame.frame:SetFrameLevel(5)
		end
		
		local checkgrp = AceGUI:Create("InlineGroup")
			checkgrp:SetTitle("")
			EventFrame:AddChild(checkgrp)
		
			local confirmedcheck = AceGUI:Create("CheckBox")
				confirmedcheck:SetLabel(L["Confirmed"])
				confirmedcheck:SetValue(true)
				checkgrp:AddChild(confirmedcheck)
			
			local acceptedcheck = AceGUI:Create("CheckBox")
				acceptedcheck:SetLabel(L["Accepted"])
				acceptedcheck:SetValue(true)
				checkgrp:AddChild(acceptedcheck)
			
			local tentativecheck = AceGUI:Create("CheckBox")
				tentativecheck:SetLabel(L["Tentative"])
				tentativecheck:SetValue(true)
				checkgrp:AddChild(tentativecheck)
				
			local standbycheck = AceGUI:Create("CheckBox")
				standbycheck:SetLabel(L["Standby"])
				standbycheck:SetValue(true)
				checkgrp:AddChild(standbycheck)
				
			local declinedcheck = AceGUI:Create("CheckBox")
				declinedcheck:SetLabel(L["Declined"])
				declinedcheck:SetValue(true)
				checkgrp:AddChild(declinedcheck)
			
			local outcheck = AceGUI:Create("CheckBox")
				outcheck:SetLabel(L["Out"])
				outcheck:SetValue(true)
				checkgrp:AddChild(outcheck)
				
			local invitedcheck = AceGUI:Create("CheckBox")
				invitedcheck:SetLabel(L["Invited"])
				invitedcheck:SetValue(true)
				checkgrp:AddChild(invitedcheck)
				
			local signedupcheck = AceGUI:Create("CheckBox")
				signedupcheck:SetLabel(L["Signed Up"])
				signedupcheck:SetValue(false)
				signedupcheck:SetDisabled(true)
				checkgrp:AddChild(signedupcheck)
			
			local tophr = AceGUI:Create("Heading")
				checkgrp:AddChild(tophr)
			
			local moderatorscheck = AceGUI:Create("CheckBox")
				moderatorscheck:SetLabel(L["Moderators Only"])
				moderatorscheck:SetValue(false)
				checkgrp:AddChild(moderatorscheck)
				
			local bottomhr = AceGUI:Create("Heading")
				checkgrp:AddChild(bottomhr)
				
			local selectallbtn = AceGUI:Create("Button")
				selectallbtn:SetText(L["Select All"])
				checkgrp:AddChild(selectallbtn)
				
			local deselectallbtn = AceGUI:Create("Button")
				deselectallbtn:SetText(L["Deselect All"])
				checkgrp:AddChild(deselectallbtn)
			
			-- checkgrp Layout
			AceGUI:RegisterLayout("checkgrp", function(content, children)
				for _, child in pairs(children) do
					child:SetWidth(content:GetWidth()/2)
				end
				children[10]:SetWidth(content:GetWidth())
				children[5]:SetPoint("TOPLEFT", children[1].frame, "TOPRIGHT", 0, 0)
				children[9]:SetWidth(content:GetWidth())
				children[9]:SetPoint("TOPLEFT", children[4].frame, "BOTTOMLEFT", 0, 0)
				children[11]:SetWidth(content:GetWidth())
				children[13]:SetPoint("TOPLEFT", children[12].frame, "TOPRIGHT", 0, 0)
			end)
			checkgrp:SetLayout("checkgrp")
			
	local mailbtn = AceGUI:Create("Button")
		mailbtn:SetText(L["Create Group"])
		EventFrame:AddChild(mailbtn)
		
	-- EventFrame callbacks
	mailbtn:SetCallback("OnClick", function()
		local attendees = {}
		for i=1, CalendarEventGetNumInvites(), 1 do
			attendees[i] = {}
			attendees[i][1],_,_,_,attendees[i][2],attendees[i][3],_,_ = CalendarEventGetInvite(i)
		end
		
		local title,_,_,_,_,_,_,_,month,day,year,hour,min,_,_,_,_,_,_,_,_,_,_,_,_ = CalendarGetEventInfo()
		

		local eventtitle = string.format("%s - %02d/%02d/%04d @ %02d:%02d", title, day, month, year, hour, min)
		local invitelist = {}
		
		for _,value in pairs(attendees) do
			if value[1] ~= playername then
				if confirmedcheck:GetValue() and value[2] == 4 then
					tinsert(invitelist, value[1])
				elseif acceptedcheck:GetValue() and value[2] == 2 then
					tinsert(invitelist, value[1])
				elseif tentativecheck:GetValue() and value[2] == 9 then
					tinsert(invitelist, value[1])
				elseif standbycheck:GetValue() and value[2] == 6 then
					tinsert(invitelist, value[1])
				elseif declinedcheck:GetValue() and value[2] == 3 then
					tinsert(invitelist, value[1])
				elseif outcheck:GetValue() and value[2] == 5 then
					tinsert(invitelist, value[1])
				elseif invitedcheck:GetValue() and value[2] == 1 then
					tinsert(invitelist, value[1])
				elseif signedupcheck:GetValue() and value[2] == 7 then
					tinsert(invitelist, value[1])
				elseif moderatorscheck:GetValue() and value[3]:len() > 0 then
					tinsert(invitelist, value[1])
				end
			end
		end
		
		local key = nil
		for index, value in pairs(loadedgroups["names"]) do
			if value == eventtitle then
				key = index
			end
		end
		
		if key then
			loadedgroups["members"][key] = invitelist
			self.db.profile.groups[eventtitle] = invitelist
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Updated group for event"].." - "..eventtitle)
		else
			loadedgroups["names"][#loadedgroups["names"] + 1] = eventtitle
			loadedgroups["members"][#loadedgroups["members"] + 1] = invitelist
			self.db.profile.groups[eventtitle] = invitelist
			if frames.AddressFrame then
				frames.AddressFrame.customdrop:SetList(loadedgroups["names"])
			end
			if frames.GroupFrame then
				frames.GroupFrame.groupms:SetItemList(loadedgroups["names"])
			end
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Created group for event"].." - "..eventtitle)
		end
	end)
	
	-- checkgrp callbacks
	moderatorscheck:SetCallback("OnValueChanged", function(widget, event, value)
		if value then
			CheckOthers(EventFrame.checkboxes, widget, false)
		end
	end)
	
	selectallbtn:SetCallback("OnClick", function()
		CheckOthers(EventFrame.checkboxes, nil, true) 
		moderatorscheck:SetValue(false)
	end)
	
	deselectallbtn:SetCallback("OnClick", function() CheckOthers(EventFrame.checkboxes, nil, false) end)
			
	-- EventFrame layout
	AceGUI:RegisterLayout("EventFrame", function(content, children)
		children[1]:ClearAllPoints()
		children[1]:SetPoint("TOPLEFT", 5, 15)
		children[1]:SetWidth(content:GetWidth())
		children[1]:SetHeight(content:GetHeight())-- - children[2].frame:GetHeight())
		children[2]:SetWidth(content:GetWidth())
		children[2]:SetPoint("TOPLEFT", children[1].frame, "BOTTOMLEFT", 0, 0)
	end)
	EventFrame:SetLayout("EventFrame")
	
	-- tooltips
	if loadedoptions["showtooltips"] then
		confirmedcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Confirmed"], L["Include members with statuses set to Confirmed."]) end)
		confirmedcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		acceptedcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Accepted"], L["Include members with statuses set to Accepted."]) end)
		acceptedcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		tentativecheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Tentative"], L["Include members with statuses set to Tentative."]) end)
		tentativecheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		standbycheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Standby"], L["Include members with statuses set to Standby."]) end)
		standbycheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		declinedcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Declined"], L["Include members with statuses set to Declined."]) end)
		declinedcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		outcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Out"], L["Include members with statuses set to Out."]) end)
		outcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		invitedcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Invited"], L["Include members with statuses set to Invited."]) end)
		invitedcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		signedupcheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Signed Up"], L["Include members with statuses set to Signed Up."]) end)
		signedupcheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		moderatorscheck:SetCallback("OnEnter", function() MM:ShowTooltip(L["Moderators Only"], L["Include only event moderators, regardless of invite status."]) end)
		moderatorscheck:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		selectallbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Select All"], L["Select all invite statuses."]) end)
		selectallbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		deselectallbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Deselect All"], L["Deselect all invite statuses."]) end)
		deselectallbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
		mailbtn:SetCallback("OnEnter", function() MM:ShowTooltip(L["Create Group"], L["Creates a new group based on the selected invite statuses."]) end)
		mailbtn:SetCallback("OnLeave", function() MM:ShowTooltip() end)
	end	
			
	-- add EventFrame children
	EventFrame.checkboxes = {}
	EventFrame.checkboxes.confirmedcheck = confirmedcheck
	EventFrame.checkboxes.acceptedcheck = acceptedcheck
	EventFrame.checkboxes.tentativecheck = tentativecheck
	EventFrame.checkboxes.standbycheck = standbycheck
	EventFrame.checkboxes.declinedcheck = declinedcheck
	EventFrame.checkboxes.outcheck = outcheck
	EventFrame.checkboxes.invitedcheck = invitedcheck
	EventFrame.checkboxes.signedupcheck = signedupcheck
	EventFrame.checkboxes.moderatorscheck = moderatorscheck
	
	-- support functions
	for _, check in pairs (EventFrame.checkboxes) do
		if check ~= moderatorscheck then
			check:SetCallback("OnValueChanged", function(widget, event, value)
				if value then
					moderatorscheck:SetValue(false)
				end
			end)
		end
	end
	
	-- add EventFrame to frames list
	frames.EventFrame = EventFrame
end

--[[--------------------
-- Mail Functions
--]]--------------------
function MM:Send()
	if frames.MessageFrame.addressbox:GetText():len() > 0 then
		if frames.MessageFrame.subject and frames.MessageFrame.subject:len() > 0 then
			message.names = BreakString(frames.MessageFrame.addressbox:GetText())
			message.subject = frames.MessageFrame.subject
			message.body = ParseBody(frames.MessageFrame.bodybox:GetText())
			
			message.niter = 1
			message.biter = getn(message.body)
			message.count = 0
			message.errors = {}
			
			local cost = getn(message.names) * getn(message.body) * 30
			
			if cost < GetMoney() then
				message.active = true
				message.time = time()
				
				SendMail(message.names[message.niter], message.subject.." ("..message.biter.." / "..getn(message.body)..")", message.body[message.biter])
				
				if loadedoptions["autoclear"] then 
					frames.MessageFrame.addressbox:SetText()
					frames.MessageFrame.addressbox:Fire("OnTextChanged", "")
					frames.MessageFrame.subjectbox:SetText()
					frames.MessageFrame.subjectbox:Fire("OnTextChanged", "")
					frames.MessageFrame.bodybox:SetText()
					if loadedoptions["insertsig"] then
						frames.MessageFrame.bodybox:SetText(loadedsigs[loadedoptions["defaultsig"]])
					end
					frames.MessageFrame.bodybox:Fire("OnTextChanged") 
				end
			else
				print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["You don't have enough money. Total Cost:"].." "..floor(abs(cost/10000)).."g"..floor(abs(mod(cost/100,100))).."s"..floor(abs(mod(cost, 100))).."c")
			end
		else
			print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please enter a subject."])
		end
	else
		print ("|cFFFFEE33"..L["MassMailer"]..": |r"..L["Please enter a recipient to mail."])
	end
end